###############################################################################
# TRAP HANDLERS
#
#   This assembly code defines handlers for the traps and interrupts understood
#   by weensyos: that is, the clock interrupt and four system calls.
#
#   There is no need to understand this code in detail!
#
###############################################################################

.text
.align 2

	.globl clockinterrupt_trap_handler
clockinterrupt_trap_handler:
	pushl $0		// error code
	pushl $32		// trap number
	jmp _generic_trap_handler

	.globl sys_yield_trap_handler
sys_yield_trap_handler:
	pushl $0
	pushl $48
	jmp _generic_trap_handler

	.globl sys_exit_trap_handler
sys_exit_trap_handler:
	pushl $0
	pushl $49
	jmp _generic_trap_handler

	.globl sys_user1_trap_handler
sys_user1_trap_handler:
	pushl $0
	pushl $50
	jmp _generic_trap_handler

	.globl sys_user2_trap_handler
sys_user2_trap_handler:
	pushl $0
	pushl $51
	jmp _generic_trap_handler
	
	.globl default_trap_handler
default_trap_handler:
	pushl $0
	pushl $500
	jmp _generic_trap_handler

_generic_trap_handler:
	# When we get here, the processor's interrupt mechanism has
	# pushed the old task status and stack registers onto the kernel stack.
	# Then one of the specific handlers pushed the trap number.
	# Now, we complete the 'registers_t' structure by pushing the extra
	# segment definitions and the general CPU registers.
	pushl %ds
	pushl %es
	pushal

	# Call the kernel's 'trap' function.
	pushl %esp
	call trap
	# 'trap' should never return.
